%%
% ===================================================================================================================== 
% This script is written by Ragna Alstadheim, May 2016.
% Solving the Smets-Wouters 2007 model, and the Lubik and Schorfheide model, 
% with optimal policy under discretion. Looping over the weight on the interest rate 
% level ("rlevel") in the loss function. Output: matrix with standard
% deviations of endogenous variables including the nominal interest rate
%% Set paths
%you need version 4.1.2 of dynare.
addpath('C:\.....\dynare\4.1.2\matlab')

%you need the "loose commitment" toolkit derived by Debortoli, Maih and Nunes, available in a local copy in the toolkit-folder inkluded in the zip-file:  
addpath('C:\......\toolkit');

%% Looping over different weights on the nominal interest rate level, Smets-Wouters (2007)
% The smets-wouters model is downloaded from the macro model database og 
ivariance=zeros(10,5);
save ivariance.mat ivariance;
for n = 1:10;
rlevel = n/200;
save n.mat n;
save parameters.mat rlevel;
dynare US_SW07_RAl % IF YOU WANT TO TEST DIFFERENT LOSS FUNCTIONS, YOU NEED TO ADJUST THE MODEL FILE
%see line 274 in model file: US_SW07_RAl.dyn
interestvar=oo_.var(1,1);
inflationvar=oo_.var(2,2);
outputvar=oo_.var(3,3);
realratevar=oo_.var(4,4);
save interestvar.mat interestvar 
save outputvar.mat outputvar
save inflationvar.mat inflationvar
save realratevar.mat realratevar
load realratevar
load interestvar 
load outputvar
load inflationvar
load n
load parameters
load ivariance
ivariance(n,:)=[interestvar^(1/2) inflationvar^(1/2) outputvar^(1/2) realratevar^(1/2) rlevel];
save ivariance.mat ivariance;

end

%% Printing results, Smets-Wouters
load ivariance
data=mat2dataset(ivariance,'VarNames',{'sdinterest', 'sdinflation', 'sdoutput', 'sdrealrate','gamma'}) 
data1=replaceWithMissing(data(:,1:4),'Numericvalues',0) 
data=horzcat(data1,data(:,5))
data_test=data
save ivariance.mat data
data

%% Looping over different weights on the nominal interest rate level, Lubik and Schorfheide (200x)
ivariance=zeros(10,5);
save ivariance.mat ivariance;
for n = 0:10;
rlevel = n/50;
save n.mat n;
save parameters.mat rlevel;
dynare LS.dyn % IF YOU WANT TO TEST DIFFERENT LOSS FUNCTIONS, YOU NEED TO ADJUST THE MODEL CODE
interestvar=oo_.var(1,1);
inflationvar=oo_.var(2,2);
outputvar=oo_.var(3,3);
realratevar=oo_.var(4,4);
save interestvar.mat interestvar 
save outputvar.mat outputvar
save inflationvar.mat inflationvar
save realratevar.mat realratevar
load realratevar
load interestvar 
load outputvar
load inflationvar
load n
load parameters
load ivariance
ivariance(n+1,:)=[interestvar^(1/2) inflationvar^(1/2) outputvar^(1/2) realratevar^(1/2) rlevel];
save ivariance.mat ivariance;
end;


%% Printing results, Lubik Schofheide
load ivariance
data=mat2dataset(ivariance,'VarNames',{'sdinterest', 'sdinflation', 'sdoutput', 'sdrealrate','gamma'}) 
data1=replaceWithMissing(data(:,1:4),'Numericvalues',0) 
data=horzcat(data1,data(:,5))
save ivariance.mat data
data


