% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This file conveniently reorganizes the output of Dynare (stored in oo_), so that it can be
% used to compute statistics, IRFs etc.

function [T,R,A0,Alead,Alag,B,iGAM0,W,SIGv,beta,gam]=GetDynareLooseCommitmentResults
global M_ oo_ options_

if isfield(M_,'orig_model')
   orig_model = M_.orig_model;
   endo_nbr = orig_model.endo_nbr;
%   endo_names = orig_model.endo_names;
   lead_lag_incidence = orig_model.lead_lag_incidence;
	exo_nbr = orig_model.exo_nbr;
	MaxLag=orig_model.maximum_lag;
	MaxLead=orig_model.maximum_lead;
else
	error([mfilename,':: Could not locate the original model'])
end

if ~isempty(strmatch('ProbabilityOfCommitment',M_.param_names))
	gam = get_param_by_name('ProbabilityOfCommitment'); % <--- 	gam  =LooseCommitmentResults.gam;
else
	gam=1;
end

beta = options_.planner_discount;

[U,Uy,W] = feval([M_.fname,'_objective_static'],zeros(endo_nbr,1),[], M_.params);
W=reshape(W,endo_nbr,endo_nbr);

klen = MaxLag + MaxLead + 1;
iyv=lead_lag_incidence';
% Find the jacobian
z = repmat(zeros(endo_nbr,1),1,klen);
z = z(nonzeros(iyv)) ;
it_ = MaxLag + 1 ;

if exo_nbr == 0
    oo_.exo_steady_state = [] ;
end
[junk,jacobia_] = feval([M_.fname '_dynamic'],z, [oo_.exo_simul oo_.exo_det_simul], M_.params, it_);
eq_nbr= size(jacobia_,1);
mult_nbr=eq_nbr;

Indices={'lag','0','lead'};
iter=1;
for j=1:numel(Indices)
    eval(['A',Indices{j},'=zeros(eq_nbr,endo_nbr);'])
	if strcmp(Indices{j},'0')||(strcmp(Indices{j},'lag') && MaxLag)||(strcmp(Indices{j},'lead') && MaxLead)
	    [junk,row,col]=find(lead_lag_incidence(iter,:)); 
	    eval(['A',Indices{j},'(:,row)=jacobia_(:,col);'])
		iter=iter+1;
	end
end
B=jacobia_(:,nnz(iyv)+1:end);

SIGv =M_.Sigma_e;

%LooseCommitmentResults=evalin('base','LooseCommitmentResults');	
%H    =LooseCommitmentResults.H;
%G    =LooseCommitmentResults.G;
Selection=oo_.dr.LooseCommitmentResults.Selection;

T=zeros(endo_nbr+mult_nbr);
T(:,Selection)=oo_.dr.ghx;
R=oo_.dr.ghu(oo_.dr.inv_order_var,:);
T=T(oo_.dr.inv_order_var,oo_.dr.inv_order_var);

iGAM0=oo_.dr.LooseCommitmentResults.iGAM0;
%%H=oo_.dr.LooseCommitmentResults.H;
%%G=oo_.dr.LooseCommitmentResults.G;