% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This files transforms the equations of the model according to the
% formulation in the paper, and then solve the problem (using
% SolveLooseCommitment.m)

% INPUTs: oo_ (structured variable generated by Dynare)
% OUTPUTs: - dr: structured variable storing the results
%          - info: information regarding the convergence


function [dr,info]=LooseCommitment(oo_)
global M_ options_ estim_params_
persistent Hold


options_ = set_default_option(options_,'noprint',0);
options_ = set_default_option(options_,'qz_criteriumLC',1.000001); %1.05 1.000001
options_= set_default_option(options_,'Reconvexify',0);
options_= set_default_option(options_,'MaxIterLC',3000);
options_= set_default_option(options_,'critLC',1e-7);

if isfield(M_,'orig_model')
    orig_model = M_.orig_model;
    M_.endo_nbr = orig_model.endo_nbr;
    M_.endo_names = orig_model.endo_names;
    M_.lead_lag_incidence = orig_model.lead_lag_incidence;
    M_.maximum_lead = orig_model.maximum_lead;
    M_.maximum_endo_lead = orig_model.maximum_endo_lead;
    M_.maximum_lag = orig_model.maximum_lag;
    M_.maximum_endo_lag = orig_model.maximum_endo_lag;
else
    M_.orig_model = M_;
end

vv=strmatch('ProbabilityOfCommitment',M_.param_names,'exact');

if ~isempty(vv)
    gam=M_.params(vv);
else
    gam=1;
end

beta = options_.planner_discount;

[U,Uy,W] = feval([M_.fname,'_objective_static'],zeros(M_.endo_nbr,1),[], M_.params);
W=reshape(W,M_.endo_nbr,M_.endo_nbr);

exo_nbr = M_.exo_nbr;
MaxLag=M_.maximum_lag;
MaxLead=M_.maximum_lead;
klen = MaxLag + MaxLead + 1;
endo_nbr = M_.endo_nbr;
endo_names = M_.endo_names;
iyv=M_.lead_lag_incidence';
% Find the jacobian
z = repmat(zeros(endo_nbr,1),1,klen);
z = z(nonzeros(iyv)) ;
it_ = MaxLag + 1 ;

if exo_nbr == 0
    oo_.exo_steady_state = [] ;
end
[junk,jacobia_] = feval([M_.fname '_dynamic'],z, [oo_.exo_simul oo_.exo_det_simul], M_.params, it_);
eq_nbr= size(jacobia_,1);
%instr_nbr=endo_nbr-eq_nbr;
mult_nbr=eq_nbr;

Indices={'lag','0','lead'};
iter=1;
for j=1:numel(Indices)
    eval(['A',Indices{j},'=zeros(eq_nbr,endo_nbr);'])
    if strcmp(Indices{j},'0')||(strcmp(Indices{j},'lag') && MaxLag)||(strcmp(Indices{j},'lead') && MaxLead)
        [junk,row,col]=find(M_.lead_lag_incidence(iter,:));
        eval(['A',Indices{j},'(:,row)=jacobia_(:,col);'])
        iter=iter+1;
    end
end
B=jacobia_(:,nnz(iyv)+1:end);

MultList=[];
for j=1:eq_nbr
    MultList=strvcat(MultList,['mult_',int2str(j)]); %#ok<VCAT>
end

%%% MAIN ENGINE %%%
Reconvexify = options_.Reconvexify;
if ~isempty(Hold)
    [H,G,iGAM0,info]=SolveLooseCommitment(Alag,A0,Alead,B,W,gam,beta,Reconvexify,Hold);
else
    [H,G,iGAM0,info]=SolveLooseCommitment(Alag,A0,Alead,B,W,gam,beta,Reconvexify);
end

if info
    dr=[];
    return
else
    Hold=H;
    % Hold=[]; use this line if persistent command is not used.
end

%%%%%%%%%%%%%%%%%%%
% H is [A0,LAMB0]=[endo_nbr,Mult_nbr]

% update the following elements
NewEndo_nbr=endo_nbr+mult_nbr;
NewEndo_names=char(endo_names,MultList);
% Update the Maximum lag right here right now
MaxLag=max(MaxLag,M_.maximum_lead); % Corrected April 15, 2010 JMA

LLI=zeros(3,NewEndo_nbr);
% current
LLI(2,:)=1;
% leads
LLI(3,1:endo_nbr)=any(Alead);
% Forward-looking multipliers
LLI(3,endo_nbr+1:end)=any(Alag');
% lags
LLI(MaxLag,:)=any(H);

LLI=LLI';
tmp=find(LLI);
LLI(tmp)=1:numel(tmp);

M_.lead_lag_incidence = LLI';
M_.maximum_lag = MaxLag;
M_.maximum_endo_lag = MaxLag;

M_.endo_nbr = NewEndo_nbr;
M_.orig_endo_nbr=M_.endo_nbr; % M_.orig_endo_nbr+mult_nbr;
M_.endo_names = NewEndo_names;
% set the state
dr=oo_.dr;
dr.ys =zeros(NewEndo_nbr,1);
dr=set_state_space(dr,M_);
order_var=dr.order_var;

T=H(order_var,order_var);
dr.ghu=G(order_var,:);
Selection=any(T);
dr.ghx=T(:,Selection);
dr.LooseCommitmentResults.Selection=Selection;
dr.LooseCommitmentResults.iGAM0=iGAM0;
