% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This files computes the impulse response functions

% INPUTs: - Periods: horizon for the impulse response functions (set to 40
% if empty)
%         - Sim_nbr: number of simulations over which to take averages (set
%         to 1000 if empty)
%         - InitShocks: vector of initial shocks (set to one std. if empty)
%         - InitVals: vector of values for initial conditions of endogenous
%         variables (set to zero if empty)
%         - CommitmentHistory: vector indicating the history of commitment
%         vs reoptimizations (drawn randomly if empty)
% OUTPUTs: - IRFs: matrix of impulse responses



function IRF=LooseCommitmentIrf(Periods,Sim_nbr,InitShocks,InitVals,CommitmentHistory)

[H,G,A0,Alead,Alag,B,iGAM0,W,SIGv,beta,gam]=GetDynareLooseCommitmentResults;
[mult_nbr,orig_endo_nbr]=size(A0);
mult_id=orig_endo_nbr+(1:mult_nbr);
endo_nbr=mult_nbr+orig_endo_nbr;
exo_nbr=size(B,2);

if nargin==4
	CommitmentHistory=[];
elseif nargin==3
	CommitmentHistory=[];
	InitVals=[];
elseif nargin==2
	CommitmentHistory=[];
	InitVals=[];
	InitShocks=[];
elseif nargin==1
	CommitmentHistory=[];
	InitVals=zeros(endo_nbr,1);
	InitShocks=[];
	Sim_nbr=1000;
elseif nargin==0
	CommitmentHistory=[];
	InitVals=[];
	InitShocks=[];
	Sim_nbr=1000;
	Periods=40;
elseif nargin >5
	error([mfilename,':: Wrong number of arguments'])
end

if isempty(InitVals)
	InitVals=zeros(endo_nbr,1);
end

IRF=LooseCommitmentIrfCore(H,G,SIGv,mult_id,gam,Periods,Sim_nbr,InitShocks,InitVals,CommitmentHistory);
IRF=mean(IRF,4);

%%for s=1:exo_nbr
%%	ifig=0;
%%	for v=1:orig_endo_nbr
%%	  if mod(v,9)==1
%%	    ifig=ifig+1;
%%	    figure('name',['Orthogonalized shocks to shock #',int2str(s),'(',int2str(ifig),')'])
%%%	    figure('name',['Orthogonalized shocks to ',M_.exo_names(s,:),'(',int2str(ifig),')'])
%%	  end
%%	  subplot(3,3,v-9*(ifig-1))
%%		vs=['variable #',int2str(v)];	% <--- 		vs=deblank(M_.endo_names(v,:));
%%		data=squeeze(IRF(v,:,s,:));
%%		subplot(3,3,v-9*(ifig-1))
%%		hp=plot([data(:,1),data(:,end),mean(data,2)],'LineWidth',2,'Color','blue');
%%		title(vs,'interpreter','none')
%%		set(hp(1:2),'LineStyle','--')
%%	end
%%end

function IRF=LooseCommitmentIrfCore(H,G,SIGv,mult_id,gam,Periods,Sim_nbr,InitShocks,InitVals,CommitmentHistory)
[endo_nbr,exo_nbr]=size(G);
IRF=nan(endo_nbr,Periods+1,exo_nbr,Sim_nbr);
CS=transpose(chol(SIGv));
G=G*CS;

for isim=1:Sim_nbr
	for s=1:exo_nbr
		YF=nan(endo_nbr,Periods+1,2);
		if ~isempty(CommitmentHistory)
			ComHist=CommitmentHistory;
		else
			ComHist=randsample([0,1],Periods,true,[1-gam,gam]);
		end
		YF(:,1,1)=InitVals;
		YF(:,1,2)=InitVals;
		if ~isempty(InitShocks)
			Shocks=InitShocks;
        else
            Shocks=randn(exo_nbr,1);
		end
        for step=1:Periods
			Y01=YF(:,step,1); Y01(mult_id)=ComHist(step)*Y01(mult_id);
			Y02=YF(:,step,2); Y02(mult_id)=ComHist(step)*Y02(mult_id);
			if step==1
				Shocks(s)=1;
			end
			YF(:,step+1,1)=H*Y01+G*Shocks;
			if step==1
				Shocks(s)=0;
			end
			YF(:,step+1,2)=H*Y02+G*Shocks;
		end
		IRF(:,:,s,isim)=YF(:,:,1)-YF(:,:,2);
	end
end
IRF=sort(IRF,4);
