% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This files computes second moments

% INPUTs: - Periods: horizon for the impulse response functions (set to 40 if empty)
%         - Sim_nbr: number of simulations over which to take averages (set to 1000 if empty)
%         - Burn: number of periods to discard in the simulation (default is 100)
%         - InitVals: vector of values for initial conditions of endogenous variables (set to zero if empty)
%         - CommitmentHistory: vector indicating the history of commitment vs reoptimizations (drawn randomly if empty)
%         - shock_sel: allows to turn o? some shocks in case the corresponding element is set to zero (default is vector of ones).
%
% OUTPUTs: - MOM: Variance covariance matrix
%          - SERIES: raw series of the simulations (not used)            
            

function [MOM, SERIES]=LooseCommitmentMoments(Periods,Sim_nbr,Burn,InitVals,CommitmentHistory, shocks_sel)
global M_

[H,G,A0,Alead,Alag,B,iGAM0,W,SIGv,beta,gam]=GetDynareLooseCommitmentResults;
[mult_nbr,orig_endo_nbr]=size(A0);
mult_id=orig_endo_nbr+(1:mult_nbr);
endo_nbr=mult_nbr+orig_endo_nbr;
[junk,exo_nbr]=size(G);

if nargin==5
    shocks_sel=ones(exo_nbr,1);
elseif nargin==4
    shocks_sel=ones(exo_nbr,1);
    CommitmentHistory=[];
elseif nargin==3
    shocks_sel=ones(exo_nbr,1);
    CommitmentHistory=[];
    InitVals=[];
elseif nargin==2
    shocks_sel=ones(exo_nbr,1);
    CommitmentHistory=[];
    InitVals=[];
    Burn=100;
elseif nargin==1
    shocks_sel=ones(exo_nbr,1);
    CommitmentHistory=[];
    InitVals=[];
    Burn=100;
    Sim_nbr=1000;
elseif nargin==0
    shocks_sel=ones(exo_nbr,1);
    CommitmentHistory=[];
    InitVals=[];
    Burn=100;
    Sim_nbr=1000;
    Periods=500;
elseif nargin >6
    error([mfilename,':: Wrong number of arguments'])
end

if isempty(InitVals)
	InitVals=zeros(endo_nbr,1);
end

CS=transpose(chol(SIGv));
G=G*CS;
MOM=0;
SERIES = nan(endo_nbr,Periods,Sim_nbr);

for isim=1:Sim_nbr
	if ~isempty(CommitmentHistory)
		ComHist=CommitmentHistory;
	else
		ComHist=randsample([0,1],Burn+Periods,true,[1-gam,gam]);      
	end
	YF=nan(endo_nbr,Periods);
	Y0=InitVals; 
	for step=1:Burn+Periods
        Shocks=randn(exo_nbr,1);
		Y0(mult_id)=ComHist(step)*Y0(mult_id);
        if step>Burn
			YF(:,step-Burn)=Y0;
            SERIES(:,step-Burn,isim) = Y0;
        end
		Y0=H*Y0+G*(Shocks.*shocks_sel);
	end
	MOM=MOM+YF*YF'/Periods;
end
MOM=MOM/Sim_nbr;

