% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This files plots the impulse response functions for 
% different degrees of commitment and two alternative histories (contained in stat and stats2)

% INPUTs: - stats, stats2: cell-array of impulse responses. The dimensions are {degree of commitment}(variables,periods,shocks) (2 alternative histories)
%         - VARNAMES: labels for the variables
%         - show_prob : vector with the grid positions of the degrees of commitment to show
%         - show_prob2 : scalar with the grid position to show for the alternative history
%         - show_shock : vectors with ones for the shocks to show
%         - prob_COM_grid: vector with the grid of commitment values (to produce the legend)


function plot_IRFs(stats,stats2,VARNAMES,show_prob, show_prob2, show_var, show_shock, probCOM_grid)
nseries = length(show_var);
Periods = size(stats{1},2);
nits_probCOM = length(show_prob);
nrow = ceil(nseries/2);
pos=0;

set(0,'DefaultAxesGridLineStyle',':');
linestyle = {'g--','b-','r-.','kx-'};

for i=1:nseries;
    SERIEStoPLOT = zeros(Periods,nits_probCOM);
    SERIEStoPLOT2 = zeros(Periods,1);
    pos=pos+1;
    for j=1:nits_probCOM
        prob_pos = find(probCOM_grid==show_prob(j));
        SERIEStoPLOT(:,j) = stats{prob_pos}(show_var(i),1:Periods,show_shock);
        if ~isempty(stats2)
            if show_prob2==show_prob(j)
                SERIEStoPLOT2 = stats2{prob_pos}(show_var(i),1:Periods,show_shock)';
            end
        end
    end
    
    SERIEStoPLOT = [SERIEStoPLOT SERIEStoPLOT2];
    
    subplot(nrow,2,pos);
    for xx = 1:size(SERIEStoPLOT,2)
        plot([0:Periods-1]', SERIEStoPLOT(1:Periods,xx),linestyle{xx},'LineWidth',1);
        hold on
    end
    axis([0 Periods-1 -Inf Inf]);
    axis 'auto y';
    grid on
    title(VARNAMES(show_var(i),:));
    hold off
end

legend([num2str(show_prob(:),'%5.1g');'Reo'],'Orientation','horizontal',...
   'Location',[0.35 0.01616 0.28 0.03213]);