% Loose Commitment in Medium- and Large-Scale Models (Debortoli, Maih, Nunes, 2010)
% Copyright: Debortoli, Maih, Nunes (2010). This version, November 2010
% Feel free to use and modify this code at your own risk.

% This files plots the impulse response functions for 
% different degrees of commitment and two alternative histories (contained in stat and stats2)

% INPUTs: - stats, array with standard deviations. The dimensions are (value of parameters,variables, degree commitment)
%         - VARNAMES: labels for the variables
%         - show_prob : vector with the grid positions of the degrees of commitment to show
%         - prob_COM_grid: vector with the grid of commitment values 

function plot_frontier(stats,VARNAMES, show_prob, show_var, probCOM_grid, mult_grid)
grid_size = size(stats,1);
nseries = length(show_var);
nits_probCOM = length(show_prob);
nrow = ceil((nseries-1)/2);
pos=0;

idplot = [1; ceil(grid_size/2); grid_size];

set(0,'DefaultAxesGridLineStyle',':');
linestyle = {'g--','b-','r-.','kx-'};

for i=1:nseries;
    SERIEStoPLOT1 = zeros(grid_size,nits_probCOM);
    SERIEStoPLOT2 = zeros(grid_size,nits_probCOM);
    
    for j=1:nits_probCOM        
        prob_pos = find(probCOM_grid==show_prob(j));
        SERIEStoPLOT1(:,j) = stats(:,show_var(1), prob_pos);
        SERIEStoPLOT2(:,j) = stats(:,show_var(i), prob_pos);
    end
    
    if i>1
        pos=pos+1;
        subplot(nrow,2,pos);
        for xx = 1:size(SERIEStoPLOT1,2)
            plot(SERIEStoPLOT1(:,xx), SERIEStoPLOT2(:,xx),linestyle{xx},'LineWidth',1);

            text(SERIEStoPLOT1(idplot,xx),SERIEStoPLOT2(idplot,xx),...
                    strcat(num2str(mult_grid(idplot)','%0.3g'),'bench'),'HorizontalAlignment','left','fontsize',8);
            hold on           
        end
        axis([-Inf Inf -Inf Inf]);
        axis 'auto x';
        axis 'auto y';
        xlabel(strcat('\sigma_{',VARNAMES(show_var(1),:),'}'))
        ylabel(strcat('\sigma_{',VARNAMES(show_var(i),:),'}'))
        grid on
        hold off
    end
end

legend([num2str(show_prob(:),'%5.1g')],'Orientation','horizontal',...
   'Location',[0.35 0.01616 0.28 0.03213]);